/*---------------------------------------------------------------------------------
 bbMemLimiter ( 2006 nocd5)
 ----------------------------------------------------------------------------------
 bbMemLimiter is a plugin for bbLean
 ----------------------------------------------------------------------------------
 bbMemLimiter is free software, released under the GNU General Public License,
 version 2, as published by the Free Software Foundation.  It is distributed
 WITHOUT ANY WARRANTY--without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  Please see the GNU General Public
 License for more details:  [http://www.fsf.org/licenses/gpl.html].
 --------------------------------------------------------------------------------*/

#include "bbMemLimiter.h"

#define LIMIT_SIZE 5000 // [K byte]
#define TIMER_INTERVAL 5000 // [ms]

//-------------------------------------------------------------------------------------

int beginPlugin(HINSTANCE hPluginInstance)
{
    WNDCLASS wc;
    ZeroMemory((void*)&wc, sizeof(wc));
    wc.lpszClassName = pluginInfo(0);
    wc.hInstance = hPluginInstance;
    wc.lpfnWndProc = PluginProc;
    RegisterClass(&wc);

    hPluginWnd = CreateWindowEx(
        WS_EX_TOOLWINDOW,
        pluginInfo(0),
        NULL,
        WS_POPUP,
        0, 0, 0, 0,
        HWND_MESSAGE,
        NULL,
        hPluginInstance,
        NULL
    );

    BBhWnd = (HWND)GetCurrentProcess();
    SendMessage(BBhWnd, BB_REGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
	TimerObject.Set();
    return 0;
}

//-------------------------------------------------------------------------------------

void endPlugin(HINSTANCE hPluginInstance)
{
    SendMessage(BBhWnd, BB_UNREGISTERMESSAGE, (WPARAM)hPluginWnd, (LPARAM)bb_messages);
    DestroyWindow(hPluginWnd);
    UnregisterClass(pluginInfo(0), hPluginInstance);
    TimerObject.Kill();
}

//-------------------------------------------------------------------------------------

LRESULT CALLBACK PluginProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_TIMER:
            PROCESS_MEMORY_COUNTERS MemInfo = TimerObject.GetMemInfo();
            if (MemInfo.WorkingSetSize/1024 > LIMIT_SIZE){
            SetProcessWorkingSetSize(BBhWnd, (SIZE_T)-1, (SIZE_T)-1);
            }
            break;
        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }
    return 0;
}

//-------------------------------------------------------------------------------------
void TimerItem::Set()
{
	SetTimer(hPluginWnd, TIMER_ID, TIMER_INTERVAL, false);
}

void TimerItem::Kill()
{
	KillTimer(hPluginWnd, TIMER_ID);
}

PROCESS_MEMORY_COUNTERS TimerItem::GetMemInfo()
{
	PROCESS_MEMORY_COUNTERS MemInfo;
	GetProcessMemoryInfo(BBhWnd, &MemInfo, sizeof(MemInfo));
	return MemInfo;
}

//-------------------------------------------------------------------------------------
LPCSTR pluginInfo (int field)
{
	switch (field)
	{
		case 0: return szVersion;       // Fallback: Plugin name + version, e.g. "MyPlugin 1.0"
		case 1:  return szAppName;       // Plugin name
		case 2:  return szInfoVersion;   // Plugin version
		case 3:  return szInfoAuthor;    // Author
		case 4:  return szInfoRelDate;   // Release date, preferably in yyyy-mm-dd format
		case 5:  return szInfoLink;      // Link to author's website
		case 6:  return szInfoEmail;     // Author's email
	}
}
